@@ -20,7 +20,7 @@ _ = gpodder.gettext
 __title__ = _('Convert video files')
 __description__ = _('Transcode video files to avi/mp4/m4v')
 __authors__ = 'Thomas Perl <thp@gpodder.org>, Bernd Schlapsi <brot@gmx.info>'
-__doc__ = 'http://wiki.gpodder.org/wiki/Extensions/VideoConverter'
+__doc__ = 'https://gpodder.github.io/docs/extensions/videoconverter.html'
 __payment__ = 'https://flattr.com/submit/auto?user_id=BerndSch&url=http://wiki.gpodder.org/wiki/Extensions/VideoConverter'
 __category__ = 'post-download'
 
