@@ -20,8 +20,8 @@
 # This metadata block gets parsed by setup.py - use single quotes only
 __tagline__ = 'Media aggregator and podcast client'
 __author__ = 'Thomas Perl <thp@gpodder.org>'
-__version__ = '3.10.15'
-__date__ = '2020-04-15'
+__version__ = '3.10.17'
+__date__ = '2020-11-23'
 __copyright__ = '© 2005-2020 The gPodder Team'
 __license__ = 'GNU General Public License, version 3 or later'
 __url__ = 'http://gpodder.org/'
@@ -84,6 +84,8 @@ del sqlite3
 
 # Is gpodder running in verbose mode?
 verbose = False
+# Is gpodder running in quiet mode?
+quiet = False
 
 
 # The User-Agent string for downloads
