@@ -149,12 +149,23 @@ class PodcastParserFeed(Feed):
         if self.max_episodes > 0 and len(entries) > self.max_episodes:
             entries = entries[:self.max_episodes]
 
+        num_duplicate_guids = 0
+
         # Search all entries for new episodes
         for entry in entries:
             episode = channel.EpisodeClass.from_podcastparser_entry(entry, channel)
             if episode is None:
                 continue
 
+            # Discard episode when its GUID collides with a newer episode
+            if episode.guid in seen_guids:
+                num_duplicate_guids += 1
+                channel._update_error = ('Discarded {} episode(s) with non-unique GUID, contact the podcast publisher to fix this issue.'
+                        .format(num_duplicate_guids))
+                logger.warn('Discarded episode with non-unique GUID, contact the podcast publisher to fix this issue. [%s] [%s]',
+                        channel.title, episode.title)
+                continue
+
             seen_guids.add(episode.guid)
             # Detect (and update) existing episode based on GUIDs
             existing_episode = existing_guids.get(episode.guid, None)
@@ -243,10 +254,11 @@ class PodcastEpisode(PodcastModelObject):
     # In theory, Linux can have 255 bytes (not characters!) in a filename, but
     # filesystems like eCryptFS store metadata in the filename, making the
     # effective number of characters less than that. eCryptFS recommends
-    # 140 chars, we use 120 here (140 - len(extension) - len(".partial")).
+    # 140 chars, we use 120 here (140 - len(extension) - len(".partial.webm"))
+    # (youtube-dl appends an extension after .partial, ".webm" is the longest).
     # References: gPodder bug 1898, http://unix.stackexchange.com/a/32834
     MAX_FILENAME_LENGTH = 120  # without extension
-    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial")  # with extension
+    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial.webm")  # with extension
 
     __slots__ = schema.EpisodeColumns
 
@@ -501,7 +513,7 @@ class PodcastEpisode(PodcastModelObject):
 
         if url is None or not os.path.exists(url):
             # FIXME: may custom downloaders provide the real url ?
-            url = registry.download_url.resolve(config, self.url, self)
+            url = registry.download_url.resolve(config, self.url, self, allow_partial)
         return url
 
     def find_unique_file_name(self, filename, extension):
